# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: innpv.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='innpv.proto',
  package='innpv.protocol',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0binnpv.proto\x12\x0einnpv.protocol\"\xcf\x01\n\nFromClient\x12\x17\n\x0fsequence_number\x18\x01 \x01(\r\x12\x37\n\ninitialize\x18\x02 \x01(\x0b\x32!.innpv.protocol.InitializeRequestH\x00\x12\x33\n\x08\x61nalysis\x18\x03 \x01(\x0b\x32\x1f.innpv.protocol.AnalysisRequestH\x00\x12/\n\x07generic\x18\x04 \x01(\x0b\x32\x1c.innpv.protocol.GenericEventH\x00\x42\t\n\x07payload\">\n\x0cGenericEvent\x12\x12\n\nevent_type\x18\x01 \x01(\t\x12\x1a\n\x12optional_arguments\x18\x02 \x01(\t\"X\n\x11InitializeRequest\x12\x18\n\x10protocol_version\x18\x01 \x01(\r\x12\x14\n\x0cproject_root\x18\x02 \x01(\t\x12\x13\n\x0b\x65ntry_point\x18\x03 \x01(\t\"F\n\x0f\x41nalysisRequest\x12\x15\n\rmock_response\x18\x01 \x01(\x08\x12\x1c\n\x14\x64\x64p_analysis_request\x18\x02 \x01(\x08\"\xca\x04\n\nFromServer\x12\x17\n\x0fsequence_number\x18\x01 \x01(\r\x12.\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.ProtocolErrorH\x00\x12\x38\n\ninitialize\x18\x03 \x01(\x0b\x32\".innpv.protocol.InitializeResponseH\x00\x12\x37\n\x0e\x61nalysis_error\x18\x05 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisErrorH\x00\x12\x38\n\nthroughput\x18\x06 \x01(\x0b\x32\".innpv.protocol.ThroughputResponseH\x00\x12\x36\n\tbreakdown\x18\x08 \x01(\x0b\x32!.innpv.protocol.BreakdownResponseH\x00\x12\x32\n\x07habitat\x18\t \x01(\x0b\x32\x1f.innpv.protocol.HabitatResponseH\x00\x12\x30\n\x06\x65nergy\x18\n \x01(\x0b\x32\x1e.innpv.protocol.EnergyResponseH\x00\x12:\n\x0butilization\x18\x0b \x01(\x0b\x32#.innpv.protocol.UtilizationResponseH\x00\x12=\n\x03\x64\x64p\x18\x0c \x01(\x0b\x32..innpv.protocol.DDPBucketSizesComputationTimesH\x00\x42\t\n\x07payloadJ\x04\x08\x04\x10\x05J\x04\x08\x07\x10\x08R\x0cmemory_usageR\x08run_time\"B\n\x17HabitatDevicePrediction\x12\x13\n\x0b\x64\x65vice_name\x18\x01 \x01(\t\x12\x12\n\nruntime_ms\x18\x02 \x01(\x02\"\x86\x01\n\x0fHabitatResponse\x12<\n\x0bpredictions\x18\x01 \x03(\x0b\x32\'.innpv.protocol.HabitatDevicePrediction\x12\x35\n\x0e\x61nalysis_error\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\"\xf1\x01\n\x0e\x45nergyResponse\x12\x19\n\x11total_consumption\x18\x01 \x01(\x02\x12>\n\ncomponents\x18\x02 \x03(\x0b\x32*.innpv.protocol.EnergyConsumptionComponent\x12\x12\n\nbatch_size\x18\x03 \x01(\x05\x12\x39\n\x11past_measurements\x18\x04 \x03(\x0b\x32\x1e.innpv.protocol.EnergyResponse\x12\x35\n\x0e\x61nalysis_error\x18\x05 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\"\x80\x01\n\x1a\x45nergyConsumptionComponent\x12\x46\n\x0e\x63omponent_type\x18\x01 \x01(\x0e\x32..innpv.protocol.EnergyConsumptionComponentType\x12\x1a\n\x12\x63onsumption_joules\x18\x02 \x01(\x02\"\x9b\x01\n\x13UtilizationResponse\x12\x31\n\x08rootNode\x18\x01 \x01(\x0b\x32\x1f.innpv.protocol.UtilizationNode\x12\x35\n\x0e\x61nalysis_error\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\x12\x1a\n\x12tensor_utilization\x18\x03 \x01(\x02\"\xd5\x01\n\x1e\x44\x44PBucketSizesComputationTimes\x12\x17\n\x0f\x66orward_time_ms\x18\x01 \x01(\x02\x12\x14\n\x0c\x62ucket_sizes\x18\x02 \x03(\x02\x12M\n\x11\x63omputation_times\x18\x03 \x03(\x0b\x32\x32.innpv.protocol.DDPExpectedMaximumComputationTimes\x12\x35\n\x0e\x61nalysis_error\x18\x04 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\"O\n\"DDPExpectedMaximumComputationTimes\x12\r\n\x05ngpus\x18\x01 \x01(\x05\x12\x1a\n\x12\x65xpected_max_times\x18\x02 \x03(\x02\"\x8c\x01\n\x12InitializeResponse\x12\x1b\n\x13server_project_root\x18\x01 \x01(\t\x12)\n\x0b\x65ntry_point\x18\x02 \x01(\x0b\x32\x14.innpv.protocol.Path\x12.\n\x08hardware\x18\x03 \x01(\x0b\x32\x1c.innpv.protocol.HardwareInfo\"[\n\rAnalysisError\x12\x15\n\rerror_message\x18\x01 \x01(\t\x12\x33\n\x0c\x66ile_context\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.FileReference\"\xa1\x02\n\x12ThroughputResponse\x12\x1a\n\x12samples_per_second\x18\x01 \x01(\x02\x12(\n predicted_max_samples_per_second\x18\x02 \x01(\x02\x12\x30\n\x0brun_time_ms\x18\x03 \x01(\x0b\x32\x1b.innpv.protocol.LinearModel\x12\x35\n\x10peak_usage_bytes\x18\x04 \x01(\x0b\x32\x1b.innpv.protocol.LinearModel\x12\x39\n\x12\x62\x61tch_size_context\x18\x05 \x01(\x0b\x32\x1d.innpv.protocol.FileReference\x12!\n\x19\x63\x61n_manipulate_batch_size\x18\x06 \x01(\x08\"\xea\x01\n\x11\x42reakdownResponse\x12\x18\n\x10peak_usage_bytes\x18\x01 \x01(\x04\x12\x1d\n\x15memory_capacity_bytes\x18\x02 \x01(\x04\x12\x1d\n\x15iteration_run_time_ms\x18\x03 \x01(\x02\x12\x12\n\nbatch_size\x18\x06 \x01(\r\x12\x35\n\x0eoperation_tree\x18\x04 \x03(\x0b\x32\x1d.innpv.protocol.BreakdownNode\x12\x32\n\x0bweight_tree\x18\x05 \x03(\x0b\x32\x1d.innpv.protocol.BreakdownNode\"\xca\x01\n\rProtocolError\x12;\n\nerror_code\x18\x01 \x01(\x0e\x32\'.innpv.protocol.ProtocolError.ErrorCode\"|\n\tErrorCode\x12\x0b\n\x07UNKNOWN\x10\x00\x12 \n\x1cUNSUPPORTED_PROTOCOL_VERSION\x10\x01\x12\x1c\n\x18UNINITIALIZED_CONNECTION\x10\x02\x12\"\n\x1e\x41LREADY_INITIALIZED_CONNECTION\x10\x03\"\x1a\n\x04Path\x12\x12\n\ncomponents\x18\x01 \x03(\t\"M\n\rFileReference\x12\'\n\tfile_path\x18\x01 \x01(\x0b\x32\x14.innpv.protocol.Path\x12\x13\n\x0bline_number\x18\x02 \x01(\r\"\xce\x01\n\rBreakdownNode\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0cnum_children\x18\x02 \x01(\r\x12/\n\x08\x63ontexts\x18\x03 \x03(\x0b\x32\x1d.innpv.protocol.FileReference\x12\x32\n\toperation\x18\x04 \x01(\x0b\x32\x1d.innpv.protocol.OperationDataH\x00\x12,\n\x06weight\x18\x05 \x01(\x0b\x32\x1a.innpv.protocol.WeightDataH\x00\x42\x06\n\x04\x64\x61ta\"{\n\x0b\x43ontextInfo\x12.\n\x07\x63ontext\x18\x01 \x01(\x0b\x32\x1d.innpv.protocol.FileReference\x12\x13\n\x0brun_time_ms\x18\x02 \x01(\x02\x12\x12\n\nsize_bytes\x18\x03 \x01(\x04\x12\x13\n\x0binvocations\x18\x04 \x01(\r\"\x83\x01\n\rOperationData\x12\x12\n\nforward_ms\x18\x01 \x01(\x02\x12\x13\n\x0b\x62\x61\x63kward_ms\x18\x02 \x01(\x02\x12\x12\n\nsize_bytes\x18\x03 \x01(\x04\x12\x35\n\x10\x63ontext_info_map\x18\x04 \x03(\x0b\x32\x1b.innpv.protocol.ContextInfo\"9\n\nWeightData\x12\x12\n\nsize_bytes\x18\x01 \x01(\x04\x12\x17\n\x0fgrad_size_bytes\x18\x02 \x01(\x04\"*\n\x0bLinearModel\x12\r\n\x05slope\x18\x01 \x01(\x01\x12\x0c\n\x04\x62ias\x18\x02 \x01(\x01\":\n\x0cHardwareInfo\x12\x10\n\x08hostname\x18\x01 \x01(\t\x12\n\n\x02os\x18\x02 \x01(\t\x12\x0c\n\x04gpus\x18\x03 \x03(\t\"\xc0\x02\n\x0fUtilizationNode\x12\x10\n\x08slice_id\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05start\x18\x03 \x01(\x04\x12\x0b\n\x03\x65nd\x18\x04 \x01(\x04\x12\x13\n\x0b\x63pu_forward\x18\x05 \x01(\x04\x12\x18\n\x10\x63pu_forward_span\x18\x06 \x01(\x04\x12\x13\n\x0bgpu_forward\x18\x07 \x01(\x04\x12\x18\n\x10gpu_forward_span\x18\x08 \x01(\x04\x12\x14\n\x0c\x63pu_backward\x18\t \x01(\x04\x12\x19\n\x11\x63pu_backward_span\x18\n \x01(\x04\x12\x14\n\x0cgpu_backward\x18\x0b \x01(\x04\x12\x19\n\x11gpu_backward_span\x18\x0c \x01(\x04\x12\x31\n\x08\x63hildren\x18\r \x03(\x0b\x32\x1f.innpv.protocol.UtilizationNode\"\x1b\n\x13MemoryUsageResponseJ\x04\x08\x01\x10\x65\"\x17\n\x0fRunTimeResponseJ\x04\x08\x01\x10\x65\"\x17\n\x0f\x41\x63tivationEntryJ\x04\x08\x01\x10\x65\"\x13\n\x0bWeightEntryJ\x04\x08\x01\x10\x65\"\x14\n\x0cRunTimeEntryJ\x04\x08\x01\x10\x65*`\n\x1e\x45nergyConsumptionComponentType\x12\x16\n\x12\x45NERGY_UNSPECIFIED\x10\x00\x12\x13\n\x0f\x45NERGY_CPU_DRAM\x10\x01\x12\x11\n\rENERGY_NVIDIA\x10\x02\x62\x06proto3'
)

_ENERGYCONSUMPTIONCOMPONENTTYPE = _descriptor.EnumDescriptor(
  name='EnergyConsumptionComponentType',
  full_name='innpv.protocol.EnergyConsumptionComponentType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ENERGY_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ENERGY_CPU_DRAM', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ENERGY_NVIDIA', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4244,
  serialized_end=4340,
)
_sym_db.RegisterEnumDescriptor(_ENERGYCONSUMPTIONCOMPONENTTYPE)

EnergyConsumptionComponentType = enum_type_wrapper.EnumTypeWrapper(_ENERGYCONSUMPTIONCOMPONENTTYPE)
ENERGY_UNSPECIFIED = 0
ENERGY_CPU_DRAM = 1
ENERGY_NVIDIA = 2


_PROTOCOLERROR_ERRORCODE = _descriptor.EnumDescriptor(
  name='ErrorCode',
  full_name='innpv.protocol.ProtocolError.ErrorCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNSUPPORTED_PROTOCOL_VERSION', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNINITIALIZED_CONNECTION', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ALREADY_INITIALIZED_CONNECTION', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2935,
  serialized_end=3059,
)
_sym_db.RegisterEnumDescriptor(_PROTOCOLERROR_ERRORCODE)


_FROMCLIENT = _descriptor.Descriptor(
  name='FromClient',
  full_name='innpv.protocol.FromClient',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sequence_number', full_name='innpv.protocol.FromClient.sequence_number', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initialize', full_name='innpv.protocol.FromClient.initialize', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis', full_name='innpv.protocol.FromClient.analysis', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='generic', full_name='innpv.protocol.FromClient.generic', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='payload', full_name='innpv.protocol.FromClient.payload',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=32,
  serialized_end=239,
)


_GENERICEVENT = _descriptor.Descriptor(
  name='GenericEvent',
  full_name='innpv.protocol.GenericEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_type', full_name='innpv.protocol.GenericEvent.event_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='optional_arguments', full_name='innpv.protocol.GenericEvent.optional_arguments', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=241,
  serialized_end=303,
)


_INITIALIZEREQUEST = _descriptor.Descriptor(
  name='InitializeRequest',
  full_name='innpv.protocol.InitializeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='protocol_version', full_name='innpv.protocol.InitializeRequest.protocol_version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_root', full_name='innpv.protocol.InitializeRequest.project_root', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entry_point', full_name='innpv.protocol.InitializeRequest.entry_point', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=305,
  serialized_end=393,
)


_ANALYSISREQUEST = _descriptor.Descriptor(
  name='AnalysisRequest',
  full_name='innpv.protocol.AnalysisRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mock_response', full_name='innpv.protocol.AnalysisRequest.mock_response', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ddp_analysis_request', full_name='innpv.protocol.AnalysisRequest.ddp_analysis_request', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=395,
  serialized_end=465,
)


_FROMSERVER = _descriptor.Descriptor(
  name='FromServer',
  full_name='innpv.protocol.FromServer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sequence_number', full_name='innpv.protocol.FromServer.sequence_number', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='innpv.protocol.FromServer.error', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initialize', full_name='innpv.protocol.FromServer.initialize', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.FromServer.analysis_error', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='throughput', full_name='innpv.protocol.FromServer.throughput', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='breakdown', full_name='innpv.protocol.FromServer.breakdown', index=5,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='habitat', full_name='innpv.protocol.FromServer.habitat', index=6,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='energy', full_name='innpv.protocol.FromServer.energy', index=7,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='utilization', full_name='innpv.protocol.FromServer.utilization', index=8,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ddp', full_name='innpv.protocol.FromServer.ddp', index=9,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='payload', full_name='innpv.protocol.FromServer.payload',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=468,
  serialized_end=1054,
)


_HABITATDEVICEPREDICTION = _descriptor.Descriptor(
  name='HabitatDevicePrediction',
  full_name='innpv.protocol.HabitatDevicePrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_name', full_name='innpv.protocol.HabitatDevicePrediction.device_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='runtime_ms', full_name='innpv.protocol.HabitatDevicePrediction.runtime_ms', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1056,
  serialized_end=1122,
)


_HABITATRESPONSE = _descriptor.Descriptor(
  name='HabitatResponse',
  full_name='innpv.protocol.HabitatResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='predictions', full_name='innpv.protocol.HabitatResponse.predictions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.HabitatResponse.analysis_error', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1125,
  serialized_end=1259,
)


_ENERGYRESPONSE = _descriptor.Descriptor(
  name='EnergyResponse',
  full_name='innpv.protocol.EnergyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='total_consumption', full_name='innpv.protocol.EnergyResponse.total_consumption', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='components', full_name='innpv.protocol.EnergyResponse.components', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='innpv.protocol.EnergyResponse.batch_size', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='past_measurements', full_name='innpv.protocol.EnergyResponse.past_measurements', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.EnergyResponse.analysis_error', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1262,
  serialized_end=1503,
)


_ENERGYCONSUMPTIONCOMPONENT = _descriptor.Descriptor(
  name='EnergyConsumptionComponent',
  full_name='innpv.protocol.EnergyConsumptionComponent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='component_type', full_name='innpv.protocol.EnergyConsumptionComponent.component_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='consumption_joules', full_name='innpv.protocol.EnergyConsumptionComponent.consumption_joules', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1506,
  serialized_end=1634,
)


_UTILIZATIONRESPONSE = _descriptor.Descriptor(
  name='UtilizationResponse',
  full_name='innpv.protocol.UtilizationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rootNode', full_name='innpv.protocol.UtilizationResponse.rootNode', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.UtilizationResponse.analysis_error', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tensor_utilization', full_name='innpv.protocol.UtilizationResponse.tensor_utilization', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1637,
  serialized_end=1792,
)


_DDPBUCKETSIZESCOMPUTATIONTIMES = _descriptor.Descriptor(
  name='DDPBucketSizesComputationTimes',
  full_name='innpv.protocol.DDPBucketSizesComputationTimes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='forward_time_ms', full_name='innpv.protocol.DDPBucketSizesComputationTimes.forward_time_ms', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bucket_sizes', full_name='innpv.protocol.DDPBucketSizesComputationTimes.bucket_sizes', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='computation_times', full_name='innpv.protocol.DDPBucketSizesComputationTimes.computation_times', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.DDPBucketSizesComputationTimes.analysis_error', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1795,
  serialized_end=2008,
)


_DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES = _descriptor.Descriptor(
  name='DDPExpectedMaximumComputationTimes',
  full_name='innpv.protocol.DDPExpectedMaximumComputationTimes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ngpus', full_name='innpv.protocol.DDPExpectedMaximumComputationTimes.ngpus', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expected_max_times', full_name='innpv.protocol.DDPExpectedMaximumComputationTimes.expected_max_times', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2010,
  serialized_end=2089,
)


_INITIALIZERESPONSE = _descriptor.Descriptor(
  name='InitializeResponse',
  full_name='innpv.protocol.InitializeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='server_project_root', full_name='innpv.protocol.InitializeResponse.server_project_root', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entry_point', full_name='innpv.protocol.InitializeResponse.entry_point', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hardware', full_name='innpv.protocol.InitializeResponse.hardware', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2092,
  serialized_end=2232,
)


_ANALYSISERROR = _descriptor.Descriptor(
  name='AnalysisError',
  full_name='innpv.protocol.AnalysisError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_message', full_name='innpv.protocol.AnalysisError.error_message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_context', full_name='innpv.protocol.AnalysisError.file_context', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2234,
  serialized_end=2325,
)


_THROUGHPUTRESPONSE = _descriptor.Descriptor(
  name='ThroughputResponse',
  full_name='innpv.protocol.ThroughputResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='samples_per_second', full_name='innpv.protocol.ThroughputResponse.samples_per_second', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='predicted_max_samples_per_second', full_name='innpv.protocol.ThroughputResponse.predicted_max_samples_per_second', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='run_time_ms', full_name='innpv.protocol.ThroughputResponse.run_time_ms', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='peak_usage_bytes', full_name='innpv.protocol.ThroughputResponse.peak_usage_bytes', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_size_context', full_name='innpv.protocol.ThroughputResponse.batch_size_context', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='can_manipulate_batch_size', full_name='innpv.protocol.ThroughputResponse.can_manipulate_batch_size', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2328,
  serialized_end=2617,
)


_BREAKDOWNRESPONSE = _descriptor.Descriptor(
  name='BreakdownResponse',
  full_name='innpv.protocol.BreakdownResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='peak_usage_bytes', full_name='innpv.protocol.BreakdownResponse.peak_usage_bytes', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='memory_capacity_bytes', full_name='innpv.protocol.BreakdownResponse.memory_capacity_bytes', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='iteration_run_time_ms', full_name='innpv.protocol.BreakdownResponse.iteration_run_time_ms', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='innpv.protocol.BreakdownResponse.batch_size', index=3,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operation_tree', full_name='innpv.protocol.BreakdownResponse.operation_tree', index=4,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weight_tree', full_name='innpv.protocol.BreakdownResponse.weight_tree', index=5,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2620,
  serialized_end=2854,
)


_PROTOCOLERROR = _descriptor.Descriptor(
  name='ProtocolError',
  full_name='innpv.protocol.ProtocolError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_code', full_name='innpv.protocol.ProtocolError.error_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PROTOCOLERROR_ERRORCODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2857,
  serialized_end=3059,
)


_PATH = _descriptor.Descriptor(
  name='Path',
  full_name='innpv.protocol.Path',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='components', full_name='innpv.protocol.Path.components', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3061,
  serialized_end=3087,
)


_FILEREFERENCE = _descriptor.Descriptor(
  name='FileReference',
  full_name='innpv.protocol.FileReference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_path', full_name='innpv.protocol.FileReference.file_path', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='line_number', full_name='innpv.protocol.FileReference.line_number', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3089,
  serialized_end=3166,
)


_BREAKDOWNNODE = _descriptor.Descriptor(
  name='BreakdownNode',
  full_name='innpv.protocol.BreakdownNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='innpv.protocol.BreakdownNode.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_children', full_name='innpv.protocol.BreakdownNode.num_children', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contexts', full_name='innpv.protocol.BreakdownNode.contexts', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operation', full_name='innpv.protocol.BreakdownNode.operation', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weight', full_name='innpv.protocol.BreakdownNode.weight', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='innpv.protocol.BreakdownNode.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=3169,
  serialized_end=3375,
)


_CONTEXTINFO = _descriptor.Descriptor(
  name='ContextInfo',
  full_name='innpv.protocol.ContextInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='context', full_name='innpv.protocol.ContextInfo.context', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='run_time_ms', full_name='innpv.protocol.ContextInfo.run_time_ms', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='innpv.protocol.ContextInfo.size_bytes', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='invocations', full_name='innpv.protocol.ContextInfo.invocations', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3377,
  serialized_end=3500,
)


_OPERATIONDATA = _descriptor.Descriptor(
  name='OperationData',
  full_name='innpv.protocol.OperationData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='forward_ms', full_name='innpv.protocol.OperationData.forward_ms', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='backward_ms', full_name='innpv.protocol.OperationData.backward_ms', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='innpv.protocol.OperationData.size_bytes', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='context_info_map', full_name='innpv.protocol.OperationData.context_info_map', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3503,
  serialized_end=3634,
)


_WEIGHTDATA = _descriptor.Descriptor(
  name='WeightData',
  full_name='innpv.protocol.WeightData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='innpv.protocol.WeightData.size_bytes', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grad_size_bytes', full_name='innpv.protocol.WeightData.grad_size_bytes', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3636,
  serialized_end=3693,
)


_LINEARMODEL = _descriptor.Descriptor(
  name='LinearModel',
  full_name='innpv.protocol.LinearModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='slope', full_name='innpv.protocol.LinearModel.slope', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bias', full_name='innpv.protocol.LinearModel.bias', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3695,
  serialized_end=3737,
)


_HARDWAREINFO = _descriptor.Descriptor(
  name='HardwareInfo',
  full_name='innpv.protocol.HardwareInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='hostname', full_name='innpv.protocol.HardwareInfo.hostname', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='os', full_name='innpv.protocol.HardwareInfo.os', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpus', full_name='innpv.protocol.HardwareInfo.gpus', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3739,
  serialized_end=3797,
)


_UTILIZATIONNODE = _descriptor.Descriptor(
  name='UtilizationNode',
  full_name='innpv.protocol.UtilizationNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='slice_id', full_name='innpv.protocol.UtilizationNode.slice_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='innpv.protocol.UtilizationNode.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start', full_name='innpv.protocol.UtilizationNode.start', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end', full_name='innpv.protocol.UtilizationNode.end', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_forward', full_name='innpv.protocol.UtilizationNode.cpu_forward', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_forward_span', full_name='innpv.protocol.UtilizationNode.cpu_forward_span', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_forward', full_name='innpv.protocol.UtilizationNode.gpu_forward', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_forward_span', full_name='innpv.protocol.UtilizationNode.gpu_forward_span', index=7,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_backward', full_name='innpv.protocol.UtilizationNode.cpu_backward', index=8,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_backward_span', full_name='innpv.protocol.UtilizationNode.cpu_backward_span', index=9,
      number=10, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_backward', full_name='innpv.protocol.UtilizationNode.gpu_backward', index=10,
      number=11, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_backward_span', full_name='innpv.protocol.UtilizationNode.gpu_backward_span', index=11,
      number=12, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='children', full_name='innpv.protocol.UtilizationNode.children', index=12,
      number=13, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3800,
  serialized_end=4120,
)


_MEMORYUSAGERESPONSE = _descriptor.Descriptor(
  name='MemoryUsageResponse',
  full_name='innpv.protocol.MemoryUsageResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4122,
  serialized_end=4149,
)


_RUNTIMERESPONSE = _descriptor.Descriptor(
  name='RunTimeResponse',
  full_name='innpv.protocol.RunTimeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4151,
  serialized_end=4174,
)


_ACTIVATIONENTRY = _descriptor.Descriptor(
  name='ActivationEntry',
  full_name='innpv.protocol.ActivationEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4176,
  serialized_end=4199,
)


_WEIGHTENTRY = _descriptor.Descriptor(
  name='WeightEntry',
  full_name='innpv.protocol.WeightEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4201,
  serialized_end=4220,
)


_RUNTIMEENTRY = _descriptor.Descriptor(
  name='RunTimeEntry',
  full_name='innpv.protocol.RunTimeEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4222,
  serialized_end=4242,
)

_FROMCLIENT.fields_by_name['initialize'].message_type = _INITIALIZEREQUEST
_FROMCLIENT.fields_by_name['analysis'].message_type = _ANALYSISREQUEST
_FROMCLIENT.fields_by_name['generic'].message_type = _GENERICEVENT
_FROMCLIENT.oneofs_by_name['payload'].fields.append(
  _FROMCLIENT.fields_by_name['initialize'])
_FROMCLIENT.fields_by_name['initialize'].containing_oneof = _FROMCLIENT.oneofs_by_name['payload']
_FROMCLIENT.oneofs_by_name['payload'].fields.append(
  _FROMCLIENT.fields_by_name['analysis'])
_FROMCLIENT.fields_by_name['analysis'].containing_oneof = _FROMCLIENT.oneofs_by_name['payload']
_FROMCLIENT.oneofs_by_name['payload'].fields.append(
  _FROMCLIENT.fields_by_name['generic'])
_FROMCLIENT.fields_by_name['generic'].containing_oneof = _FROMCLIENT.oneofs_by_name['payload']
_FROMSERVER.fields_by_name['error'].message_type = _PROTOCOLERROR
_FROMSERVER.fields_by_name['initialize'].message_type = _INITIALIZERESPONSE
_FROMSERVER.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_FROMSERVER.fields_by_name['throughput'].message_type = _THROUGHPUTRESPONSE
_FROMSERVER.fields_by_name['breakdown'].message_type = _BREAKDOWNRESPONSE
_FROMSERVER.fields_by_name['habitat'].message_type = _HABITATRESPONSE
_FROMSERVER.fields_by_name['energy'].message_type = _ENERGYRESPONSE
_FROMSERVER.fields_by_name['utilization'].message_type = _UTILIZATIONRESPONSE
_FROMSERVER.fields_by_name['ddp'].message_type = _DDPBUCKETSIZESCOMPUTATIONTIMES
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['error'])
_FROMSERVER.fields_by_name['error'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['initialize'])
_FROMSERVER.fields_by_name['initialize'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['analysis_error'])
_FROMSERVER.fields_by_name['analysis_error'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['throughput'])
_FROMSERVER.fields_by_name['throughput'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['breakdown'])
_FROMSERVER.fields_by_name['breakdown'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['habitat'])
_FROMSERVER.fields_by_name['habitat'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['energy'])
_FROMSERVER.fields_by_name['energy'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['utilization'])
_FROMSERVER.fields_by_name['utilization'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['ddp'])
_FROMSERVER.fields_by_name['ddp'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_HABITATRESPONSE.fields_by_name['predictions'].message_type = _HABITATDEVICEPREDICTION
_HABITATRESPONSE.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_ENERGYRESPONSE.fields_by_name['components'].message_type = _ENERGYCONSUMPTIONCOMPONENT
_ENERGYRESPONSE.fields_by_name['past_measurements'].message_type = _ENERGYRESPONSE
_ENERGYRESPONSE.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_ENERGYCONSUMPTIONCOMPONENT.fields_by_name['component_type'].enum_type = _ENERGYCONSUMPTIONCOMPONENTTYPE
_UTILIZATIONRESPONSE.fields_by_name['rootNode'].message_type = _UTILIZATIONNODE
_UTILIZATIONRESPONSE.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_DDPBUCKETSIZESCOMPUTATIONTIMES.fields_by_name['computation_times'].message_type = _DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES
_DDPBUCKETSIZESCOMPUTATIONTIMES.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_INITIALIZERESPONSE.fields_by_name['entry_point'].message_type = _PATH
_INITIALIZERESPONSE.fields_by_name['hardware'].message_type = _HARDWAREINFO
_ANALYSISERROR.fields_by_name['file_context'].message_type = _FILEREFERENCE
_THROUGHPUTRESPONSE.fields_by_name['run_time_ms'].message_type = _LINEARMODEL
_THROUGHPUTRESPONSE.fields_by_name['peak_usage_bytes'].message_type = _LINEARMODEL
_THROUGHPUTRESPONSE.fields_by_name['batch_size_context'].message_type = _FILEREFERENCE
_BREAKDOWNRESPONSE.fields_by_name['operation_tree'].message_type = _BREAKDOWNNODE
_BREAKDOWNRESPONSE.fields_by_name['weight_tree'].message_type = _BREAKDOWNNODE
_PROTOCOLERROR.fields_by_name['error_code'].enum_type = _PROTOCOLERROR_ERRORCODE
_PROTOCOLERROR_ERRORCODE.containing_type = _PROTOCOLERROR
_FILEREFERENCE.fields_by_name['file_path'].message_type = _PATH
_BREAKDOWNNODE.fields_by_name['contexts'].message_type = _FILEREFERENCE
_BREAKDOWNNODE.fields_by_name['operation'].message_type = _OPERATIONDATA
_BREAKDOWNNODE.fields_by_name['weight'].message_type = _WEIGHTDATA
_BREAKDOWNNODE.oneofs_by_name['data'].fields.append(
  _BREAKDOWNNODE.fields_by_name['operation'])
_BREAKDOWNNODE.fields_by_name['operation'].containing_oneof = _BREAKDOWNNODE.oneofs_by_name['data']
_BREAKDOWNNODE.oneofs_by_name['data'].fields.append(
  _BREAKDOWNNODE.fields_by_name['weight'])
_BREAKDOWNNODE.fields_by_name['weight'].containing_oneof = _BREAKDOWNNODE.oneofs_by_name['data']
_CONTEXTINFO.fields_by_name['context'].message_type = _FILEREFERENCE
_OPERATIONDATA.fields_by_name['context_info_map'].message_type = _CONTEXTINFO
_UTILIZATIONNODE.fields_by_name['children'].message_type = _UTILIZATIONNODE
DESCRIPTOR.message_types_by_name['FromClient'] = _FROMCLIENT
DESCRIPTOR.message_types_by_name['GenericEvent'] = _GENERICEVENT
DESCRIPTOR.message_types_by_name['InitializeRequest'] = _INITIALIZEREQUEST
DESCRIPTOR.message_types_by_name['AnalysisRequest'] = _ANALYSISREQUEST
DESCRIPTOR.message_types_by_name['FromServer'] = _FROMSERVER
DESCRIPTOR.message_types_by_name['HabitatDevicePrediction'] = _HABITATDEVICEPREDICTION
DESCRIPTOR.message_types_by_name['HabitatResponse'] = _HABITATRESPONSE
DESCRIPTOR.message_types_by_name['EnergyResponse'] = _ENERGYRESPONSE
DESCRIPTOR.message_types_by_name['EnergyConsumptionComponent'] = _ENERGYCONSUMPTIONCOMPONENT
DESCRIPTOR.message_types_by_name['UtilizationResponse'] = _UTILIZATIONRESPONSE
DESCRIPTOR.message_types_by_name['DDPBucketSizesComputationTimes'] = _DDPBUCKETSIZESCOMPUTATIONTIMES
DESCRIPTOR.message_types_by_name['DDPExpectedMaximumComputationTimes'] = _DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES
DESCRIPTOR.message_types_by_name['InitializeResponse'] = _INITIALIZERESPONSE
DESCRIPTOR.message_types_by_name['AnalysisError'] = _ANALYSISERROR
DESCRIPTOR.message_types_by_name['ThroughputResponse'] = _THROUGHPUTRESPONSE
DESCRIPTOR.message_types_by_name['BreakdownResponse'] = _BREAKDOWNRESPONSE
DESCRIPTOR.message_types_by_name['ProtocolError'] = _PROTOCOLERROR
DESCRIPTOR.message_types_by_name['Path'] = _PATH
DESCRIPTOR.message_types_by_name['FileReference'] = _FILEREFERENCE
DESCRIPTOR.message_types_by_name['BreakdownNode'] = _BREAKDOWNNODE
DESCRIPTOR.message_types_by_name['ContextInfo'] = _CONTEXTINFO
DESCRIPTOR.message_types_by_name['OperationData'] = _OPERATIONDATA
DESCRIPTOR.message_types_by_name['WeightData'] = _WEIGHTDATA
DESCRIPTOR.message_types_by_name['LinearModel'] = _LINEARMODEL
DESCRIPTOR.message_types_by_name['HardwareInfo'] = _HARDWAREINFO
DESCRIPTOR.message_types_by_name['UtilizationNode'] = _UTILIZATIONNODE
DESCRIPTOR.message_types_by_name['MemoryUsageResponse'] = _MEMORYUSAGERESPONSE
DESCRIPTOR.message_types_by_name['RunTimeResponse'] = _RUNTIMERESPONSE
DESCRIPTOR.message_types_by_name['ActivationEntry'] = _ACTIVATIONENTRY
DESCRIPTOR.message_types_by_name['WeightEntry'] = _WEIGHTENTRY
DESCRIPTOR.message_types_by_name['RunTimeEntry'] = _RUNTIMEENTRY
DESCRIPTOR.enum_types_by_name['EnergyConsumptionComponentType'] = _ENERGYCONSUMPTIONCOMPONENTTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FromClient = _reflection.GeneratedProtocolMessageType('FromClient', (_message.Message,), {
  'DESCRIPTOR' : _FROMCLIENT,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.FromClient)
  })
_sym_db.RegisterMessage(FromClient)

GenericEvent = _reflection.GeneratedProtocolMessageType('GenericEvent', (_message.Message,), {
  'DESCRIPTOR' : _GENERICEVENT,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.GenericEvent)
  })
_sym_db.RegisterMessage(GenericEvent)

InitializeRequest = _reflection.GeneratedProtocolMessageType('InitializeRequest', (_message.Message,), {
  'DESCRIPTOR' : _INITIALIZEREQUEST,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.InitializeRequest)
  })
_sym_db.RegisterMessage(InitializeRequest)

AnalysisRequest = _reflection.GeneratedProtocolMessageType('AnalysisRequest', (_message.Message,), {
  'DESCRIPTOR' : _ANALYSISREQUEST,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.AnalysisRequest)
  })
_sym_db.RegisterMessage(AnalysisRequest)

FromServer = _reflection.GeneratedProtocolMessageType('FromServer', (_message.Message,), {
  'DESCRIPTOR' : _FROMSERVER,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.FromServer)
  })
_sym_db.RegisterMessage(FromServer)

HabitatDevicePrediction = _reflection.GeneratedProtocolMessageType('HabitatDevicePrediction', (_message.Message,), {
  'DESCRIPTOR' : _HABITATDEVICEPREDICTION,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.HabitatDevicePrediction)
  })
_sym_db.RegisterMessage(HabitatDevicePrediction)

HabitatResponse = _reflection.GeneratedProtocolMessageType('HabitatResponse', (_message.Message,), {
  'DESCRIPTOR' : _HABITATRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.HabitatResponse)
  })
_sym_db.RegisterMessage(HabitatResponse)

EnergyResponse = _reflection.GeneratedProtocolMessageType('EnergyResponse', (_message.Message,), {
  'DESCRIPTOR' : _ENERGYRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.EnergyResponse)
  })
_sym_db.RegisterMessage(EnergyResponse)

EnergyConsumptionComponent = _reflection.GeneratedProtocolMessageType('EnergyConsumptionComponent', (_message.Message,), {
  'DESCRIPTOR' : _ENERGYCONSUMPTIONCOMPONENT,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.EnergyConsumptionComponent)
  })
_sym_db.RegisterMessage(EnergyConsumptionComponent)

UtilizationResponse = _reflection.GeneratedProtocolMessageType('UtilizationResponse', (_message.Message,), {
  'DESCRIPTOR' : _UTILIZATIONRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.UtilizationResponse)
  })
_sym_db.RegisterMessage(UtilizationResponse)

DDPBucketSizesComputationTimes = _reflection.GeneratedProtocolMessageType('DDPBucketSizesComputationTimes', (_message.Message,), {
  'DESCRIPTOR' : _DDPBUCKETSIZESCOMPUTATIONTIMES,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.DDPBucketSizesComputationTimes)
  })
_sym_db.RegisterMessage(DDPBucketSizesComputationTimes)

DDPExpectedMaximumComputationTimes = _reflection.GeneratedProtocolMessageType('DDPExpectedMaximumComputationTimes', (_message.Message,), {
  'DESCRIPTOR' : _DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.DDPExpectedMaximumComputationTimes)
  })
_sym_db.RegisterMessage(DDPExpectedMaximumComputationTimes)

InitializeResponse = _reflection.GeneratedProtocolMessageType('InitializeResponse', (_message.Message,), {
  'DESCRIPTOR' : _INITIALIZERESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.InitializeResponse)
  })
_sym_db.RegisterMessage(InitializeResponse)

AnalysisError = _reflection.GeneratedProtocolMessageType('AnalysisError', (_message.Message,), {
  'DESCRIPTOR' : _ANALYSISERROR,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.AnalysisError)
  })
_sym_db.RegisterMessage(AnalysisError)

ThroughputResponse = _reflection.GeneratedProtocolMessageType('ThroughputResponse', (_message.Message,), {
  'DESCRIPTOR' : _THROUGHPUTRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ThroughputResponse)
  })
_sym_db.RegisterMessage(ThroughputResponse)

BreakdownResponse = _reflection.GeneratedProtocolMessageType('BreakdownResponse', (_message.Message,), {
  'DESCRIPTOR' : _BREAKDOWNRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.BreakdownResponse)
  })
_sym_db.RegisterMessage(BreakdownResponse)

ProtocolError = _reflection.GeneratedProtocolMessageType('ProtocolError', (_message.Message,), {
  'DESCRIPTOR' : _PROTOCOLERROR,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ProtocolError)
  })
_sym_db.RegisterMessage(ProtocolError)

Path = _reflection.GeneratedProtocolMessageType('Path', (_message.Message,), {
  'DESCRIPTOR' : _PATH,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.Path)
  })
_sym_db.RegisterMessage(Path)

FileReference = _reflection.GeneratedProtocolMessageType('FileReference', (_message.Message,), {
  'DESCRIPTOR' : _FILEREFERENCE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.FileReference)
  })
_sym_db.RegisterMessage(FileReference)

BreakdownNode = _reflection.GeneratedProtocolMessageType('BreakdownNode', (_message.Message,), {
  'DESCRIPTOR' : _BREAKDOWNNODE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.BreakdownNode)
  })
_sym_db.RegisterMessage(BreakdownNode)

ContextInfo = _reflection.GeneratedProtocolMessageType('ContextInfo', (_message.Message,), {
  'DESCRIPTOR' : _CONTEXTINFO,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ContextInfo)
  })
_sym_db.RegisterMessage(ContextInfo)

OperationData = _reflection.GeneratedProtocolMessageType('OperationData', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONDATA,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.OperationData)
  })
_sym_db.RegisterMessage(OperationData)

WeightData = _reflection.GeneratedProtocolMessageType('WeightData', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTDATA,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.WeightData)
  })
_sym_db.RegisterMessage(WeightData)

LinearModel = _reflection.GeneratedProtocolMessageType('LinearModel', (_message.Message,), {
  'DESCRIPTOR' : _LINEARMODEL,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.LinearModel)
  })
_sym_db.RegisterMessage(LinearModel)

HardwareInfo = _reflection.GeneratedProtocolMessageType('HardwareInfo', (_message.Message,), {
  'DESCRIPTOR' : _HARDWAREINFO,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.HardwareInfo)
  })
_sym_db.RegisterMessage(HardwareInfo)

UtilizationNode = _reflection.GeneratedProtocolMessageType('UtilizationNode', (_message.Message,), {
  'DESCRIPTOR' : _UTILIZATIONNODE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.UtilizationNode)
  })
_sym_db.RegisterMessage(UtilizationNode)

MemoryUsageResponse = _reflection.GeneratedProtocolMessageType('MemoryUsageResponse', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYUSAGERESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.MemoryUsageResponse)
  })
_sym_db.RegisterMessage(MemoryUsageResponse)

RunTimeResponse = _reflection.GeneratedProtocolMessageType('RunTimeResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNTIMERESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.RunTimeResponse)
  })
_sym_db.RegisterMessage(RunTimeResponse)

ActivationEntry = _reflection.GeneratedProtocolMessageType('ActivationEntry', (_message.Message,), {
  'DESCRIPTOR' : _ACTIVATIONENTRY,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ActivationEntry)
  })
_sym_db.RegisterMessage(ActivationEntry)

WeightEntry = _reflection.GeneratedProtocolMessageType('WeightEntry', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTENTRY,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.WeightEntry)
  })
_sym_db.RegisterMessage(WeightEntry)

RunTimeEntry = _reflection.GeneratedProtocolMessageType('RunTimeEntry', (_message.Message,), {
  'DESCRIPTOR' : _RUNTIMEENTRY,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.RunTimeEntry)
  })
_sym_db.RegisterMessage(RunTimeEntry)


# @@protoc_insertion_point(module_scope)
